VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "clsPerformance"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

' ------------------------------------------------------------------
'                   Klassenmodul clsPerformance
' ------------------------------------------------------------------
'
' Zweck:  Sehr genaue Zeitmessungen in Millisekunden durch
'         hochauflsende Performance-Tiemr
'
' Stand:  11/98, Tips&Tricks der BasicPro 6/98
'
' ------------------------------------------------------------------
'                        ffentliche Elemente:
' ------------------------------------------------------------------
'
' Methoden:
'
'  - StartTimer
'    Startet die Zhlung
'
'  - StopTimer
'    Beendet die Zhlung
'
' Funktionen:
'
'  - Average
'    Liefert einen Durschnittswert fr ein Parameter-Array azus
'    mehreren einzelnen Mewerten
'
' Eigenschaften:
'
'  - Supported
'    Informiert ber die Verfgbarkeit eines Performance-Timers
'
'  - Frequency
'    Frequenz des Timers (Auflsung)
'
'  - TimerOffset
'    Gibt den ungefhren Offset zurck, den der VB-Code des
'    Klassenmoduls selber fr die Bearbeitung bentigt
'
' ------------------------------------------------------------------
'           Copyright  1998 Mathias Schiffer fr BasicPro
'             BasicPro Online: http://www.basicworld.com
' ------------------------------------------------------------------


' -----
' TYPEN
' -----
Private Type PRIV_T_LARGE_INTEGER
  Lo As Long
  Hi As Long
End Type

' --------------
' API-FUNKTIONEN
' --------------
Private Declare Function QueryPerformanceCounter Lib "kernel32" (lpPerformanceCount As PRIV_T_LARGE_INTEGER) As Long
Private Declare Function QueryPerformanceFrequency Lib "kernel32" (lpFrequency As PRIV_T_LARGE_INTEGER) As Long

' --------
' VARIABLE
' --------
Private mvarPerfFrequency As PRIV_T_LARGE_INTEGER
Private mvarCounterStart As PRIV_T_LARGE_INTEGER
Private mvarCounterStop As PRIV_T_LARGE_INTEGER
Private mvarFrequency As Double
Private mvarSupported As Boolean


' ------------------------------------------------------------------


Public Sub StartTimer()
' Startet den PerformanceCounter
  QueryPerformanceCounter mvarCounterStart
End Sub


Public Sub StopTimer()
' Stoppt den PerformanceCounter
  QueryPerformanceCounter mvarCounterStop
End Sub


Property Get Timer() As Double
' Ermittelt das letzte Counter-Intervall
Dim crStart As Currency, crStop As Currency

  ' LargeIntegers in Currency umwandeln
  crStart = LargeInt2Curr(mvarCounterStart)
  crStop = LargeInt2Curr(mvarCounterStop)
  
  ' Ergebnis (in Millisekunden) unter Beruecksichtigung der Frequenz
  Timer = ((crStop - crStart) / mvarFrequency) * 1000#
  
End Property


Public Function Average(ParamArray Times() As Variant) As Double
Dim i As Integer, Total As Currency

  ' Durchschnitt mehrerer erhaltener Werte ermitteln
  For i = LBound(Times) To UBound(Times)
    Total = Total + Times(i)
  Next i

  Average = Total / (UBound(Times()) - LBound(Times))

End Function


Public Property Get Supported() As Boolean
    Supported = mvarSupported
End Property


Public Property Get Frequency() As Double
  Frequency = CDbl(mvarFrequency)
End Property


Public Property Get TimerOffset() As Double
' Ermittelt nherungsweise die Verzgerung fr die Methoden StartTimer und StopTimer
Dim i As Integer, Total As Currency

  For i = 1 To 100
    Me.StartTimer
    Me.StopTimer
    Total = Total + LargeInt2Curr(mvarCounterStop) - LargeInt2Curr(mvarCounterStart)
  Next i

  TimerOffset = CDbl(((Total / 100) / mvarFrequency) * 1000#)

End Property


Private Sub Class_Initialize()
Dim lResp As Long

  ' Frequenz feststellen und prfen, ob PerformanceCounter untersttzt wird
  mvarSupported = CBool(QueryPerformanceFrequency(mvarPerfFrequency))
  mvarFrequency = LargeInt2Curr(mvarPerfFrequency)
       
End Sub


Private Function LargeInt2Curr(LargeInt As PRIV_T_LARGE_INTEGER) As Currency
' Wandelt einen PRIV_TLARGEINTEGER in einen Currency-Datentyp um

  If LargeInt.Lo > 0 Then
    LargeInt2Curr = LargeInt.Lo
  Else
    LargeInt2Curr = CCur(2 ^ 31) + CCur(LargeInt.Lo And &H7FFFFFFF)
  End If

  LargeInt2Curr = LargeInt2Curr + LargeInt.Hi * CCur(2 ^ 32)

End Function

